angular.module('khlm.controllers', ['khlm.conf', 'khlm.services', 'ionic.utils'])

.controller('AppController', function($scope, $rootScope, $ionicModal, $ionicPopup, $ionicLoading, $http, $localstorage, appconfig, feedmanager, $window, $filter) {
  // Form data for the syncUrls modal
  $scope.loginData = {};

  // Create the syncUrls modal that we will use later
  $ionicModal.fromTemplateUrl('templates/syncform.html', {
    scope: $scope
  }).then(function(modal) {
    $scope.modal = modal;
  });

  // Triggered in the syncUrls modal to close it
  $scope.closeLogin = function() {
    $scope.modal.hide();
  };

  // Open the syncUrls modal
  $scope.syncUrls = function() {
    $scope.modal.show();
  };
  
  $scope.lastsync = $localstorage.get("urls_updated", "never");
  console.log($window.localStorage);

  // Perform the syncUrls action when the user submits the form
  $scope.doSyncUrls = function() {
    console.log('Doing sync', $scope.loginData);

    $ionicLoading.show({
      template: 'Connecting to UCLL portal...'
    });
    $http.get(appconfig.url_shibproxy + "/", {
      params: {
        "l": $scope.loginData.login,
        "p": $scope.loginData.password
      }
    }).success(function(data) {
      $ionicLoading.hide();
      console.log(data);
      var errors = [];
      var updatedCount = 0;
      if (!data.loggedIn) {
        errors.push("Could not login to KHLeuven portal.");
      } else {
        if (data.feeds && data.feeds.notices) {
          $localstorage.set('url_notices', data.feeds.notices);
          ++updatedCount;
        } else {
          errors.push("Could not find your notices RSS feed.");
        }
        if (data.feeds && data.feeds.projectwork) {
          $localstorage.set('url_projectwork', data.feeds.projectwork);
          ++updatedCount;
        } else {
          errors.push("Could not find the project work Atom feed.");
        }
        if (data.feeds && data.feeds.internat) {
          $localstorage.set('url_internat', data.feeds.internat);
          ++updatedCount;
        } else {
          errors.push("Could not find the international news RSS feed.");
        }
        if (data.icals.coursecal && data.icals.coursecal) {
          $localstorage.set('url_coursecal', data.icals.coursecal);
          ++updatedCount;
        } else {
          errors.push("Could not find your personal course calendar iCal feed.")
        }
      }
      if (updatedCount > 0) {
        var date = new Date;
        var date_f = $filter('date')(date, "dd/MM/yyyy HH:mm");
        $localstorage.set('urls_updated', date_f);
        feedmanager.updateUrls();
        $scope.lastsync = date_f;
      }
      if (errors.length) {
        var lis = errors.join('</li><li class="item item-text-wrap item-err">');
        $ionicPopup.alert({
          title: 'Could not retrieve all data',
          template: '<ul class="list"><li class="item item-text-wrap item-err">'+lis+'</li></ul>'
        });
      } else {
        $ionicPopup.alert({
          title: 'Success',
          template: '<div class="card"><div class="item item-text-wrap item-ok">'+'All feed URLs have been updated successfully.'+'</div></div>'
        });
      }
      $scope.loginData = {};
      $scope.closeLogin();
      if ($rootScope.refresher) {
        $rootScope.refresher(true);
      }
    }).error(function(data) {
      $ionicLoading.hide();
      console.log("ERROR: " + data);
    });
  };
  
  $scope.init = function() {
    feedmanager.onUpdatedUrls(function() {
      $scope.customFeeds = feedmanager.getFeeds().custom;
      $scope.customIcals = feedmanager.getIcals().custom;
      console.log("init: onupdatedurls");
    });
    feedmanager.updateUrls();
  };
})

.controller("FeedController", function($http, $scope, $rootScope, $stateParams, $location, $ionicScrollDelegate, $ionicViewService, appconfig, feedmanager) {
  $scope.init = function() {
    console.log("FeedController::init()");
  };
  
  $scope.$on('$ionicView.afterEnter', function() {
    console.log("FeedController::afterEnter()");
    $scope.doRefresh(true);
    $rootScope.refresher = $scope.doRefresh;
  });
  
  $scope.doRefresh = function(showLoader) {
    $scope.loading = showLoader;
    $ionicScrollDelegate.scrollTop(true);
    
    var feedId = $stateParams.feedId;
    var feeds = feedmanager.getFeeds();
    console.log(feeds);
    console.log(feedId);
    var feedConf = feeds[feedId];
    if (!feedConf) {
      feedConf = feeds.custom[feedId];
      if (!feedConf) {
        $ionicViewService.nextViewOptions({disableBack: true});
        $location.path('/app/feed/notices');
      }
    }
    $scope.title = feedConf.title;
    $http.get(appconfig.url_feed_parser + "/", {
      params: {
        url: feedConf.feed
      }
    }).success(function(data) {
      console.log(data);
      if (data.responseData && data.valid) {
        $scope.rssTitle = data.responseData.feed.title;
        $scope.rssUrl = data.responseData.feed.feedUrl;
        $scope.rssSiteUrl = data.responseData.feed.link;
        $scope.entries = data.responseData.feed.entries;
      } else {
        $scope.rssTitle = 'Could not load';
        $scope.rssUrl = '';
        $scope.rssSiteUrl = '';
        $scope.entries = [{
          title: 'Could not load',
          contentSnippet: 'Could not load RSS feed.',
          content: 'Could not load RSS feed.'
        }];
      }
      $scope.loading = false;
      $scope.$broadcast('scroll.refreshComplete');
    }).error(function(data) {
      console.log("ERROR: " + data);
      $scope.loading = false;
      $scope.$broadcast('scroll.refreshComplete');
    });
  };
  
  $scope.expandhtml = function(entry, targetType) {
    if (targetType == 'a')
      return;
    if (entry.expanded) {
      entry.contentSnippet = entry.origContentSnippet;
      entry.expanded = false;
    } else {
      entry.origContentSnippet = entry.contentSnippet;
      entry.contentSnippet = entry.content;
      entry.expanded = true;
    }
    $ionicScrollDelegate.resize();
  }
})

.controller("CalendarController", function($http, $scope, $rootScope, $stateParams, $location, $ionicScrollDelegate, $ionicViewService, appconfig, feedmanager) {
  $scope.init = function() {
    console.log("CalendarController::init()");
  }
  
  $scope.$on('$ionicView.afterEnter', function() {
    console.log("CalendarController::afterEnter()");
    $scope.doRefresh(true);
    $rootScope.refresher = $scope.doRefresh;
  });
  
  $scope.doRefresh = function(showLoader) {
    $scope.loading = showLoader;
    $ionicScrollDelegate.scrollTop(true);
    
    var icalId = $stateParams.icalId;
    var icals = feedmanager.getIcals();
    var icalConf = icals[icalId];
    if (!icalConf) {
      icalConf = icals.custom[icalId];
      if (!icalConf) {
        $ionicViewService.nextViewOptions({disableBack: true});
        $location.path('/app/calendar/coursecal');
      }
    }
    $scope.title = icalConf.title;
    $http.get(appconfig.url_ical_parser + "/", {
      params: {
        url: icalConf.feed
      } 
    }).success(function(data) {
      console.log(data);
      if (!data || !data.cal || !data.cal.VEVENT) {
        $scope.rssTitle = 'Could not load';
        $scope.rssUrl = '';
        $scope.rssSiteUrl = '';
        $scope.entries = [{
          title: 'Could not load',
          hourStart: '',
          hourEnd: '',
          location: 'Could not load iCal feed.',
          content: 'Could not load iCal feed.'
        }];
        $scope.loading = false;
        $scope.$broadcast('scroll.refreshComplete');
      } else {
        var entries = [];
        var prevstart = false;
        var prevfuture = false;
        var groupWithPrevious = false;
        var id = 1;
        var scrollToId = false;
        
        // Sort it
        data.cal.VEVENT.sort(function(a, b) {
          var astart = moment(a.DTSTART, "YYYYMMDDTHHmmssZ");
          var bstart = moment(b.DTSTART, "YYYYMMDDTHHmmssZ");
          return astart.isBefore(bstart) ? -1 : 1;
        });
        
        for (var i in data.cal.VEVENT) {
          var e = data.cal.VEVENT[i];
          var start = moment(e.DTSTART, "YYYYMMDDTHHmmssZ");
          var fstart = start.format("HH:mm");
          var fday = start.format("dddd DD MMMM YYYY");
          var end = moment(e.DTEND, "YYYYMMDDTHHmmssZ");
          var fend = end.format("HH:mm");
          var hourStart = fstart;
          var hourEnd = fend;
          var location = e.LOCATION;
          var title = e.SUMMARY || e.DESCRIPTION;
          var onlySummaryOrDescription = !(e.SUMMARY && e.DESCRIPTION);
          var description = onlySummaryOrDescription ? '' : (e.DESCRIPTION || e.SUMMARY);
          var future = end.isAfter(moment());
          if (future && !prevfuture) {
            scrollToId = id;
          }
          prevfuture = future;
          if (prevstart) {
            groupWithPrevious = start.isSame(prevstart, 'day');
          }
          prevstart = start;
          if (!groupWithPrevious) {
            entries.push({title: fday, title: fday, extracls: 'item-divider'});
          }
          entries.push({id: id, start: start, title: title, description: description, location: location, hourStart: hourStart, hourEnd: hourEnd, extracls: future ? 'future' : 'past'});
          ++id;
        }
        
        $scope.entries = entries;
        if (scrollToId) {
          $scope.loading = false;
          $location.hash("cal-entry-" + scrollToId);
          console.log($location.hash());
          setTimeout(function() {
            $ionicScrollDelegate.anchorScroll(true);
          }, 0);
        }
      }
    }).error(function(data) {
      console.log("ERROR: " + data);
      $scope.loading = false;
      $scope.$broadcast('scroll.refreshComplete');
    });
  }
})

.controller("BustimesController", function($http, $scope) {
  $scope.init = function() {
  }
})

.controller("SettingsController", function($http, $scope, $localstorage, $location, $ionicViewService, $ionicModal, $ionicPopup, feedmanager, $window) {
  $scope.closeUrlForm = function() {
    $scope.modal.hide();
  };

  $scope.showUrlForm = function() {
    var title = 'Custom URL';
    if ($scope.customUrlType == 'feed')
      title = 'Custom feed URL';
    if ($scope.customUrlType == 'ical')
      title = 'Custom iCalendar URL';
    $scope.modal.title = title;
    $scope.modal.show();
  };
  
  $scope.addUrl = function(type) {
    $scope.customUrlId = 0;
    $scope.customUrlNew = true;
    $scope.customUrlType = type;
    $scope.urlData = {
      title: '',
      url: ''
    };
    $scope.showUrlForm();
  };
  
  $scope.editUrl = function(item, index, targetType) {
    console.log(targetType);
    if (targetType != 'div')
      return;
    
    $scope.customUrlIndex = index;
    $scope.customUrlNew = false;
    $scope.customUrlType = item.type;
    $scope.urlData = {
      title: item.title,
      feed: item.feed
    };
    $scope.showUrlForm();
  };
  
  $scope.doSaveUrl = function() {
    if (!$scope.urlData || !$scope.urlData.title || !$scope.urlData.feed) {
      $ionicPopup.alert({
        title: 'Fill in all fields.',
        template: 'You have to fill in both the URL and a descriptive title.'
      });
      return;
    }
    
    var list = false;
    if ($scope.customUrlType == 'feed')
      list = $scope.customFeeds;
    else if ($scope.customUrlType == 'ical')
      list = $scope.customCals;
    if (list) {
      var item = {
        title: $scope.urlData.title,
        feed: $scope.urlData.feed,
        type: $scope.customUrlType
      };
      if ($scope.customUrlNew) {
        list.push(item);
      } else {
        list[$scope.customUrlIndex] = item;
      }
    }
    $scope.closeUrlForm();
  };
    
  $scope.init = function() {
    console.log("SettingsController::init()");
    $scope.settings = {};
    $scope.customFeeds = [];
    $scope.customCals = [];
    
    // Form data for the custom feed/calendar modal
    $scope.urlData = {};

    // Create the syncUrls modal that we will use later
    $ionicModal.fromTemplateUrl('templates/urlform.html', {
      scope: $scope
    }).then(function(modal) {
      $scope.modal = modal;
    });
  };
  
  $scope.save = function() {
    $localstorage.set('url_notices', $scope.settings.url_notices);
    $localstorage.set('url_projectwork', $scope.settings.url_projectwork);
    $localstorage.set('url_internat', $scope.settings.url_internat);
    $localstorage.set('url_coursecal', $scope.settings.url_coursecal);
    
    // Custom URLs
    $localstorage.setObject('custom_feeds', $scope.customFeeds);
    $localstorage.setObject('custom_cals', $scope.customCals);
    
    feedmanager.updateUrls();
    $ionicViewService.nextViewOptions({disableBack: true});
    $location.path('/app/calendar/coursecal'); // Go to course calendar
  }
  
  $scope.reorderItem = function(item, fromIndex, toIndex) {
    var list = false;
    if (item.type == 'feed')
      list = $scope.customFeeds;
    else if (item.type == 'ical')
      list = $scope.customCals;
    if (list) {
      list.splice(fromIndex, 1);
      list.splice(toIndex, 0, item);
    }
  }
  
  $scope.deleteItem = function(item) {
    var list = false;
    if (item.type == 'feed')
      list = $scope.customFeeds;
    else if (item.type == 'ical')
      list = $scope.customCals;
    if (list) {
      list.splice(list.indexOf(item), 1);
    }
  }
  
  $scope.$on('$ionicView.afterEnter', function() {
    console.log("SettingsController::afterEnter()");
    $scope.showDelete = true;
    $scope.showReorder = true;
    $scope.settings.url_notices = feedmanager.getFeeds().notices.feed;
    $scope.settings.url_projectwork = feedmanager.getFeeds().projectwork.feed;
    $scope.settings.url_internat = feedmanager.getFeeds().internat.feed;
    $scope.settings.url_coursecal = feedmanager.getIcals().coursecal.feed;
    $scope.customFeeds = feedmanager.getFeeds().custom;
    $scope.customCals = feedmanager.getIcals().custom;
  });
})

